import java.io.*;
import javax.security.*;
import HTTPClient.*;

/**
 * Client does not maintain persistent connection with SeSe,
 * so the protocol goes something like I send to the server
 * ApplicationID, Object crypted with server's public key
 * when I receive the Object I decrypt it, using the client's 
 * private key, and get the Server dummy ID ( for the sake of 
 * maintaining consistency ), and the return Object, at the 
 * time of application registration, the client generates the 
 * key pair keeps the private key with it self and broadcasts 
 * public key to the server, now the applications which have 
 * access to this particular key, and know the application 
 * registration id will be able to submit the requests, and 
 * receive replies.
 * clientsk refers to the client's secret key
 * serverpk refers to the server's public key
 *
 * @author Sandeep K Srivastava (sandy@cse.iitb.ernet.in)
 * @vendor SeSe technologies IIT Bombay
 */

public class CipherClient {
   public static final String 
      keyPairName = "RSA",
   cipherName   = "RSA/ECB/OneAndZeroes";

   /** 
    * Stores the path of client's secret key
    */
   public String clientSkFile = null;

   public static final int strength=512;
   /** 
    * server's Public Key, received at the time of registration of package
    */
   protected PublicKey serverPk     =null;
   /** 
    * client's Private key, will be stored in the file system, will be
    * readable by packages with certain privileges
    */
   protected PrivateKey clientSk    =null;
   /** 
    * client's ID
    */
   protected String clientID        =null;
   /**
    * Cipher class we use
    */
   protected Cipher rsa             =null;
   /** 
    * server's ID
    */
   protected String serverID        =null;
   /**
    * takes clientID, clientSkFile as a parameter, and loads the client's
    * secret key and server public key from the given files
    */
   public CipherClient(String clientID, String clientSkFile, String serverPkFile) 
      throws FileNotFoundException, SecurityException {
         this.clientID = clientID;
         try {
            // clientsk / serverpk can be set by the application if it has the instance
            // and wan't to avoid the load time over and over again
            if (clientSk==null) {
               FileInputStream   fl  =new FileInputStream(clientSkFile);
               ObjectInputStream ois =new ObjectInputStream(fl);
               clientSk              =(PrivateKey)ois.readObject();
            }
            if (serverPk==null) {
               FileInputStream   fl  =new FileInputStream(serverPkFile);
               ObjectInputStream ois =new ObjectInputStream(fl);
               serverPk              =(PublicKey)ois.readObject();
            }
         } catch (Exception e) {
            e.printStackTrace();
         }
      }

   protected static ObjectInputStream getObjectInputStream(byte [] inbuf)
      throws IOException {
         ByteArrayInputStream bais = new ByteArrayInputStream(inbuf);
         return new ObjectInputStream(bais);
      }

   public byte [] getCryptedStream(Object ob) throws Exception {
         ByteArrayOutputStream baos = new ByteArrayOutputStream();
         ObjectOutputStream oos = new ObjectOutputStream(baos);
         oos.writeObject(clientID);
         oos.writeObject(ob);
         oos.close();
         byte [] plain = baos.toByteArray();
         rsa = Cipher.getInstance(cipherName);
         rsa.initEncrypt(serverPk);
         return rsa.crypt(plain);
   }
   //                                         -1
   // gets the serverID, performs clientsk  [-] to get the Object, and Login.
   //
   public Object getDecryptedObject(byte[] mes) throws Exception {
      Object ret;
      rsa = Cipher.getInstance(cipherName);
      rsa.initDecrypt(clientSk);
      byte [] combo = rsa.crypt(mes);
      ObjectInputStream ois = getObjectInputStream(combo);
      serverID = (String)ois.readObject();
      ret=ois.readObject();
      ois.close();
      return ret;
   }
   /** 
    * Generates the private key and the public key for the client and stores them in file
    * clientSecretKey, and clientPublicKey
    * clientPublicKey          is uploaded to the server
    * while clientSecretKey    is stored in the file system at some secure place
    */
   public static void main (String args[]) {
      try {
         System.out.println("CipherClient: generating Public and Private Key for Application");
         KeyPairGenerator kg = KeyPairGenerator.getInstance(keyPairName);
         kg.initialize(strength, new SecureRandom());
         KeyPair pair = kg.generateKeyPair();
         PublicKey  clientPubk = pair.getPublic();
         PrivateKey clientSeck = pair.getPrivate();
         FileOutputStream   fl  =new FileOutputStream("clientSecretKey");
         ObjectOutputStream oos =new ObjectOutputStream(fl);
         oos.writeObject(clientSeck);
         fl                     =new FileOutputStream("clientPublicKey");
         oos                    =new ObjectOutputStream(fl);
         oos.writeObject(clientPubk);
         System.out.println("CipherClient: Please upload the clientPublicKey to the server"
               +" before you begin with Programming");
      }
      catch ( Exception anyex ) {
         anyex.printStackTrace();
         System.err.println("CipherClient: Unable to generate the client private and public keys");
      }
   }
}
